"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Transformer = void 0;
const graphql_transformer_core_1 = require("@aws-amplify/graphql-transformer-core");
const graphql_transformer_common_1 = require("graphql-transformer-common");
const create_send_email_lambda_1 = require("./create-send-email-lambda");
const aws_iam_1 = require("@aws-cdk/aws-iam");
const STACK_NAME = 'SendEmailTransformer';
const directiveName = "sendEmail";
class Transformer extends graphql_transformer_core_1.TransformerPluginBase {
    constructor() {
        super('amplify-graphql-send-email-transformer', 
        /* GraphQL */ `
          directive @${directiveName}(template:Template, trigger:String="INSERT") on OBJECT
          input Template {
            subject: String!
            bodyText: String!
            bodyHtml: String!
            sender: String!
            recipient: String!
          }
        `);
        this.object = (definition, directive, ctx) => {
            validateModelDirective(definition);
            const directiveArguments = getDirectiveArguments(directive);
            validateDirectiveArguments(directiveArguments, definition);
            this.directiveObjectTypeDefinitions.push({
                node: definition,
                fieldName: definition.name.value,
                fieldParams: directiveArguments,
            });
        };
        this.generateResolvers = (context) => {
            const stack = context.stackManager.createStack(STACK_NAME);
            // lambda role
            const role = new aws_iam_1.Role(stack, `${STACK_NAME}LambdaRole`, {
                assumedBy: new aws_iam_1.ServicePrincipal('lambda.amazonaws.com'),
            });
            // create lambda inputs
            const fieldMappings = this.directiveObjectTypeDefinitions.reduce((acc, def) => {
                const table = getTable(context, def.node);
                const ddbTable = table;
                acc[ddbTable.tableArn] = def.fieldParams;
                return acc;
            }, {});
            // create lambda function
            const lambda = (0, create_send_email_lambda_1.createLambda)(stack, context.api.host, role, fieldMappings);
            // // creates event source mapping for each table
            createSourceMappings(this.directiveObjectTypeDefinitions, context, lambda);
        };
        this.directiveObjectTypeDefinitions = [];
    }
}
exports.Transformer = Transformer;
const getDirectiveArguments = (directive) => {
    const directiveWrapped = new graphql_transformer_core_1.DirectiveWrapper(directive);
    return directiveWrapped.getArguments({
        trigger: "INSERT",
        template: {
            subject: "",
            bodyText: "",
            bodyHtml: "",
            sender: "",
            recipient: "",
        },
    });
};
const validateModelDirective = (object) => {
    const modelDirective = object.directives.find((dir) => dir.name.value === "model");
    if (!modelDirective) {
        throw new graphql_transformer_core_1.InvalidDirectiveError(`Types annotated with @${directiveName} must also be annotated with @model.`);
    }
};
const validateDirectiveArguments = (directiveArguments, object) => {
    const isTriggerValid = ["INSERT", "MODIFY", "REMOVE"].includes(directiveArguments.trigger);
    if (!isTriggerValid) {
        throw new graphql_transformer_core_1.InvalidDirectiveError(`Types annotated with @${directiveName} only supports "INSERT", "MODIFY", "REMOVE" as triggers.`);
    }
};
const createSourceMappings = (typeDefinitions, context, lambda) => {
    const stack = context.stackManager.getStack(STACK_NAME);
    for (const def of typeDefinitions) {
        const type = def.node.name.value;
        const indexName = context.resourceHelper.getModelNameMapping(type);
        const table = getTable(context, def.node);
        const ddbTable = table;
        ddbTable.grantStreamRead(lambda.role);
        (0, create_send_email_lambda_1.createEventSourceMapping)(stack, indexName, lambda, ddbTable.tableStreamArn);
    }
};
const getTable = (context, definition) => {
    const ddbDataSource = context.dataSources.get(definition);
    const tableName = graphql_transformer_common_1.ModelResourceIDs.ModelTableResourceID(definition.name.value);
    const table = ddbDataSource.ds.stack.node.findChild(tableName);
    return table;
};
